/*
 *  Copyright (c) 2006 Copenhagen University Astronomical Observatory
 *  All possible rights reserved
 *	Filename:	keyboard.cpp	
 *  Abstract:	General keyboard handling routines	
 *  Author:		Jeppe Jønch Andersen (jja@astro.ku.dk)
 *  Revision:
 *  Remarks:
 */
 
#include "keyboard.h"
#include <termios.h>
#include <unistd.h>   
#include <sys/time.h>
#include <sys/types.h>
#include <sys/select.h> 
#include <stdio.h>

static termios term;
static bool initialized = false;

int _kbhit (void)
{   // Set no delay
    static struct timeval tv = {0, 0};
    fd_set rdfs;

    // See if key has been pressed
    /*FD_ZERO (&rdfs);
    FD_SET  (STDERR_FILENO, &rdfs);

    if (select  (STDERR_FILENO + 1, &rdfs, NULL, NULL, &tv) <= 0)
        return 0;

    if (FD_ISSET (STDERR_FILENO, &rdfs))
        return 1;

    return 0;*/
    FD_ZERO (&rdfs);
    FD_SET  (STDIN_FILENO, &rdfs);
    return select(STDIN_FILENO + 1, &rdfs, NULL, NULL, &tv);
}

int _mygetch (void)
{
    char ch = -1;

    if ( read (STDIN_FILENO, &ch, 1) <= 0)
        return -1;

    return ch;
}

void keyboard_init(void) 
{
	termios new_term;
	tcgetattr(STDIN_FILENO, &term);
	new_term = term;
	new_term.c_lflag &= ~ICANON;
	new_term.c_lflag &= ~ECHO;
	tcsetattr(STDIN_FILENO, TCSANOW, &new_term);
	setbuf(stdin, NULL);
	initialized = true;
}

void keyboard_close(void)
{
	if( initialized ){
		tcsetattr(STDIN_FILENO, TCSANOW, &term);
		initialized = false;
	}
}

// Set keyboard to raw mode so getch will work
void keyboard_enable_raw ()
{
    // set to non canonical mode, echo off, ignore signals
    struct termios current;
    // save current terminal settings
    tcgetattr (STDERR_FILENO, &current);

    // set to non canonical mode, echo off, ignore signals
    term = current;
    current.c_lflag &= ~(ECHO | ICANON | IEXTEN);
    current.c_cc[VMIN] = 1;
    current.c_cc[VTIME] = 0;
    tcsetattr (STDERR_FILENO, TCSAFLUSH, &current);
}

void keyboard_disable_raw ()
{
    // Restore old terminal settings
    tcsetattr (STDERR_FILENO, TCSAFLUSH, &term);
}

///////////////////////////////////////////////////////////////////////////////
// EOF
