FUNCTION SIGCLIP_AVG, x , l_rej , h_rej , nit=nit , RMS = rms
;+
; NAME:
;
;       SIGCLIP_AVG
;
; PURPOSE:
;
;       returns sigma clipped average of numeric array
;
; CALLING SEQUENCE:
;
;       A = SIGCLIP_AVG( X , L_REJ , H_REJ , [ NIT , RMS = rms ] )
;
; INPUTS:
;
;       X		Array for which to calculate average.
;
;       L_REJ		Low sigma clipping limit.
;
;       H_REJ		High sigma clipping limit.
;
; OPTIONAL KEYWORDS:
;
;       NIT		Number of sigma clipping iterations. Default is 2.
;
; OUTPUT:
;
;	A		Sigma clipped average 
;
; OPTIONAL OUTPUTS:
;
;	RMS		Is returned as sigma clipped RMS.
;
; RESTRICTIONS
;
; 	Use non-standard IDL procedure IDL_TYPE
;
; MODIFICATION HISTORY:
;       WRITTEN, Michael Andersen CUOBS, December, 1995
;       Slight mod ANS Mar 99
;-

  ON_ERROR, 2

; check that x is a numeric array

  IF ( (WHERE( [ 0 , 7 , 8 ] EQ IDL_TYPE( X ) ))(0) NE -1 ) THEN $
  MESSAGE, 'Input array must be numeric'

  IF ( N_ELEMENTS( x ) EQ 1 ) THEN MESSAGE, "Variable must be an array"

  IF ( NOT KEYWORD_SET( NIT ) ) THEN nit = 2

; Get robust first estimate by median

  m	= MEDIAN( x )
  s	= STDDEV( x )
  q	= WHERE( x GT m - l_rej * s AND x LT m + h_rej * s, count )
  if count ne 0 then begin
  m	= MEAN( x(q) )
  rms	= STDDEV( x(q) )

; Then iterate

  FOR i = 1 , nit DO BEGIN
    p	= WHERE(     x(q) GT m - l_rej * rms     $
	         AND x(q) LT m + h_rej * rms , k )
    ;print,'***',k
    IF( k LT 2 ) THEN RETURN, m
    q	= q(p)
    m	= MEAN( x(q) )
    rms	= STDDEV( x(q) )
  ENDFOR

 endif else begin
  m = mean(x)
 endelse

  RETURN, m

END
