FUNCTION SIGMA_CLP, a, clip=clip, niterat=niterat 
;+
; NAME:
;       SIGMA_CLP
;
; PURPOSE:
;	Remove strongly deviating points from an array by replacing
;	values deviating CLIP times the standard deviation of the array
;	by the median of the array. This can be repeated NITERAT times.
;
; CALLING SEQUENCE:
;	result = SIGMA_CLP( input_array , clip = number [, niterat = number])
;	E.G.:  
;	result = SIGMA_CLP( a, clip = 3, niterat = 2)
;
; INPUTS:
;	INPUT_VECTOR	An array of floating point data
;
;	CLIP	Defines the threshold for replacing data
;		deviating by more than CLIP times the standard 
;		deviation from the average of the array.
;
; OPTIONAL INPUTS:
;	NITERAT Repeats the sigma-clipping process NITERAT times.
;		Is by default set to 1.
;
; OUTPUT:
;	result	The sigma-clipped array, where deviant data are
;		replaced by the median of the array.
;
; SIDE EFFECTS
;	None.
;
; MODIFICATION HISTORY:
; 	WRITTEN 
;	Febuary 1996 by Anton Norup Sorensen (norup@astro.ku.dk)
;	Modified June 1997 ANS
;
;-
 b=a ;Avoid affecting original data
 replaceval=median(b)
 sigma=stddev(b)
 idx=where(abs(b-replaceval) gt clip*sigma, count)
 if count gt 0 then b(idx)=replaceval

 ;print,'Bad points filtered out:',count,'  Sigma:',sigma,'   Clip:',clip,'  Niterat:',niterat

 if niterat gt 1 then b=sigma_clp(b,clip=clip,niterat=niterat-1)

 RETURN,b

END
